from enum import Enum
from .twiz_state_manager import TwizStateManager


class StateType(Enum):
    PREV = 'last_state'
    CURR = 'current_state'


class TurnState:
    state_manager: TwizStateManager
    state_type: StateType

    def __init__(self, state_manager: TwizStateManager, state_type: StateType = StateType.CURR):
        self.state_manager = state_manager
        self.state_type = state_type

    @property
    def state(self) -> dict:
        if self.state_type == StateType.PREV:
            return self.state_manager.last_state
        elif self.state_type == StateType.CURR:
            return self.state_manager.current_state
        else:
            raise Exception("Unrecognized State")

    @property
    def text(self) -> str:
        return self.state.get('text', None) if self.state else None

    def set_text(self, text: str = None):
        self.state_manager.write_state_attribute('text', text)

    @property
    def intent(self) -> str:
        return self.state.get('intent', None) if self.state else None

    def set_intent(self, intent: str = None):
        self.state_manager.write_state_attribute('intent', intent)

    @property
    def flow_state(self) -> str:
        return self.state.get('flow_state', None) if self.state else None

    def set_flow_state(self, flow_state: str = None):
        self.state_manager.write_state_attribute('flow_state', flow_state)

    @property
    def current_method(self) -> int:
        return self.state.get('current_method', 0) if self.state else 0

    def set_current_method(self, current_method: int = 0):
        self.state_manager.write_state_attribute('current_method', current_method)

    @property
    def current_step(self) -> int:
        return self.state.get('current_step', 0) if self.state else 0

    def set_current_step(self, current_step: int = 0):
        self.state_manager.write_state_attribute('current_step', current_step)

    @property
    def choose_responder(self) -> str:
        return self.state.get('choose_responder', None) if self.state else None

    def set_choose_responder(self, choose_responder: str = None):
        self.state_manager.write_state_attribute('choose_responder', choose_responder)

    @property
    def data_source(self) -> str:
        return self.state.get('data_source', None) if self.state else None

    def set_data_source(self, data_source: str = None):
        self.state_manager.write_state_attribute('data_source', data_source)

    @property
    def identify_process_responder(self) -> str:
        return self.state.get('identify_process_responder', None) if self.state else None

    def set_identify_process_responder(self, identify_process_responder: str = None):
        self.state_manager.write_state_attribute('identify_process_responder', identify_process_responder)

    @property
    def evi_qa_responder(self) -> str:
        return self.state.get('evi_qa_responder', None) if self.state else None

    def set_evi_qa_responder(self, evi_qa_responder: str = None):
        self.state_manager.write_state_attribute('evi_qa_responder', evi_qa_responder)

    @property
    def fallback_responder(self) -> str:
        return self.state.get('fallback_responder', None) if self.state else None

    def set_fallback_responder(self, fallback_responder: str = None):
        self.state_manager.write_state_attribute('fallback_responder', fallback_responder)

    @property
    def is_experiment(self) -> bool:
        return self.state.get('is_experiment', False) if self.state else False

    def set_is_experiment(self, is_experiment: bool = False):
        self.state_manager.write_state_attribute('is_experiment', is_experiment)

    @property
    def is_factoid_question(self) -> bool:
        return self.state.get('is_factoid_question', False) if self.state else False

    def set_is_factoid_question(self, is_factoid_question: bool = False):
        self.state_manager.write_state_attribute('is_factoid_question', is_factoid_question)

    @property
    def launch_screen_selected_tile(self) -> int:
        return self.state.get('launch_screen_selected_tile', 0) if self.state else 0

    def set_launch_screen_selected_tile(self, launch_screen_selected_tile: int = 0):
        self.state_manager.write_state_attribute('launch_screen_selected_tile', launch_screen_selected_tile)

    @property
    def launch_responder(self) -> str:
        return self.state.get('launch_responder', None) if self.state else None

    def set_launch_responder(self, launch_responder: str = None):
        self.state_manager.write_state_attribute('launch_responder', launch_responder)

    @property
    def list_item_selected(self) -> int:
        return self.state.get('list_item_selected', 0) if self.state else 0

    def set_list_item_selected(self, list_item_selected: int = 0):
        self.state_manager.write_state_attribute('list_item_selected', list_item_selected)

    @property
    def query_result_ids(self) -> list:
        return self.state.get('query_result_ids', None) if self.state else None

    def set_query_result_ids(self, query_result_ids: list = None):
        self.state_manager.write_state_attribute('query_result_ids', query_result_ids)

    @property
    def recipe_basic_qa(self) -> str:
        return self.state.get('recipe_basic_qa', None) if self.state else None

    def set_recipe_basic_qa(self, recipe_basic_qa: str = None):
        self.state_manager.write_state_attribute('recipe_basic_qa', recipe_basic_qa)

    @property
    def sensitive(self) -> dict:
        return self.state.get('sensitive', None) if self.state else None

    def set_sensitive(self, sensitive: dict = None):
        self.state_manager.write_state_attribute('sensitive', sensitive)

    @property
    def sensitive_responder(self) -> str:
        return self.state.get('sensitive_responder', None) if self.state else None

    def set_sensitive_responder(self, sensitive_responder: str = None):
        self.state_manager.write_state_attribute('sensitive_responder', sensitive_responder)

    @property
    def show_steps_responder(self) -> str:
        return self.state.get('show_steps_responder', None) if self.state else None

    def set_show_steps_responder(self, show_steps_responder: str = None):
        self.state_manager.write_state_attribute('show_steps_responder', show_steps_responder)

    @property
    def task_chosen(self) -> dict:
        return self.state.get('task_chosen', None) if self.state else None

    def set_task_chosen(self, task_chosen: dict = None):
        self.state_manager.write_state_attribute('task_chosen', task_chosen)

    @property
    def task_classifier_result(self) -> str:
        return self.state.get('task_classifier_result', None) if self.state else None

    def set_task_classifier_result(self, task_classifier_result: str = None):
        self.state_manager.write_state_attribute('task_classifier_result', task_classifier_result)

    @property
    def task_complete_responder(self) -> str:
        return self.state.get('task_complete_responder', None) if self.state else None

    def set_task_complete_responder(self, task_complete_responder: str = None):
        self.state_manager.write_state_attribute('task_complete_responder', task_complete_responder)

    @property
    def task_step_detail_responder(self) -> str:
        return self.state.get('task_step_detail_responder', None) if self.state else None

    def set_task_step_detail_responder(self, task_step_detail_responder: str = None):
        self.state_manager.write_state_attribute('task_step_detail_responder', task_step_detail_responder)

    @property
    def user_event(self) -> dict:
        return self.state.get('user_event', None) if self.state else None

    def set_user_event(self, user_event: dict = None):
        self.state_manager.write_state_attribute('user_event', user_event)

    @property
    def user_event_identify_process_responder(self) -> str:
        return self.state.get('user_event_identify_process_responder', None) if self.state else None

    def set_user_event_identify_process_responder(self, user_event_identify_process_responder: str = None):
        self.state_manager.write_state_attribute('user_event_identify_process_responder',
                                                 user_event_identify_process_responder)

    @property
    def visible_components(self) -> dict:
        return self.state.get('visible_components', None) if self.state else None

    def set_visible_components(self, visible_components: dict = None):
        self.state_manager.write_state_attribute('visible_components', visible_components)

    @property
    def visible_components_clean(self) -> dict:
        return self.state.get('visible_components_clean', None) if self.state else None

    def set_visible_components_clean(self, visible_components_clean: dict = None):
        self.state_manager.write_state_attribute('visible_components_clean', visible_components_clean)

    @property
    def yes_no_responder(self) -> str:
        return self.state.get('yes_no_responder', None) if self.state else None

    def set_yes_no_responder(self, yes_no_responder: str = None):
        self.state_manager.write_state_attribute('yes_no_responder', yes_no_responder)

    @property
    def viewport(self) -> dict:
        return self.state.get('viewport', None) if self.state else None

    def set_viewport(self, viewport: dict = None):
        self.state_manager.write_state_attribute('viewport', viewport)

    @property
    def features(self) -> dict:
        return self.state.get('features', None) if self.state else None

    def set_features(self, features: dict = None):
        self.state_manager.write_state_attribute('features', features)

    @property
    def candidate_responses(self) -> dict:
        return self.state.get('candidate_responses', None) if self.state else None

    def set_candidate_responses(self, candidate_responses: dict = None):
        self.state_manager.write_state_attribute('candidate_responses', candidate_responses)

    @property
    def evi_rr_response(self) -> bool:
        return self.state.get('evi_rr_response', False) if self.state else False

    def set_evi_rr_response(self, evi_rr_response: bool = False):
        self.state_manager.write_state_attribute('evi_rr_response', evi_rr_response)

    @property
    def number_searches(self) -> int:
        return self.state.get('number_searches', 0) if self.state else 0

    def set_number_searches(self, number_searches: int = 0):
        self.state_manager.write_state_attribute('number_searches', number_searches)

    @property
    def response(self) -> str:
        return self.state.get('response', None) if self.state else None

    def set_response(self, response: str = None):
        self.state_manager.write_state_attribute('response', response)

    @property
    def directives(self) -> str:
        return self.state.get('directives', None) if self.state else None

    def set_directives(self, directives: str = None):
        self.state_manager.write_state_attribute('directives', directives)

    @property
    def a_b_test(self) -> any:
        return self.state.get('a_b_test', None) if self.state else None

    def set_a_b_test(self, a_b_test: any = None):
        self.state_manager.write_state_attribute('a_b_test', a_b_test)

    @property
    def in_game(self) -> bool:
        return self.state.get('in_game', False) if self.state else False

    def set_in_game(self, in_game: bool = False):
        self.state_manager.write_state_attribute('in_game', in_game)

    @property
    def previous_negative(self) -> bool:
        return self.state.get('previous_negative', False) if self.state else False

    def set_previous_negative(self, previous_negative: bool = False):
        self.state_manager.write_state_attribute('previous_negative', previous_negative)

    @property
    def yes_no_category(self) -> str:
        return self.state.get('yes_no_category', None) if self.state else None

    def set_yes_no_category(self, yes_no_category: str = None):
        self.state_manager.write_state_attribute('yes_no_category', yes_no_category)

    @property
    def is_all_ingredients(self) -> bool:
        return self.state.get('is_all_ingredients', False) if self.state else False

    def set_is_all_ingredients(self, is_all_ingredients: bool = False):
        self.state_manager.write_state_attribute('is_all_ingredients', is_all_ingredients)

    @property
    def go_back_in_video(self) -> bool:
        return self.state.get('go_back_in_video', False) if self.state else False

    def set_go_back_in_video(self, go_back_in_video: bool = False):
        self.state_manager.write_state_attribute('go_back_in_video', go_back_in_video)

    @property
    def wikihow_faq_response_index(self) -> int:
        return self.state.get('wikihow_faq_response_index', 0) if self.state else 0

    def set_wikihow_faq_response_index(self, wikihow_faq_response_index: int = 0):
        self.state_manager.write_state_attribute('wikihow_faq_response_index', wikihow_faq_response_index)

    @property
    def general_faq_response_index(self) -> int:
        return self.state.get('general_faq_response_index', 0) if self.state else 0

    def set_general_faq_response_index(self, general_faq_response_index: int = 0):
        self.state_manager.write_state_attribute('general_faq_response_index', general_faq_response_index)

    @property
    def read_fact(self) -> bool:
        return self.state.get('read_fact', False) if self.state else False

    def set_read_fact(self, read_fact: bool = False):
        self.state_manager.write_state_attribute('read_fact', read_fact)

    @property
    def input_offensive(self) -> str:
        return self.state.get('input_offensive', None) if self.state else None

    def set_input_offensive(self, input_offensive: str = None):
        self.state_manager.write_state_attribute('input_offensive', input_offensive)

    # ----- variables which can only be read, not set ----- #
    @property
    def is_apl_supported(self) -> bool:
        supported_interfaces = self.state.get('supported_interfaces', None) if self.state else None
        if supported_interfaces:
            return supported_interfaces.get('apl', False)
        else:
            return False

    @property
    def creation_date_time(self) -> str:
        return self.state.get('creation_date_time', None) if self.state else None

    @property
    def app_id(self) -> str:
        return self.state.get('app_id', None) if self.state else None

    @property
    def ask_access_token(self) -> str:
        return self.state.get('ask_access_token', None) if self.state else None

    @property
    def asr(self) -> str:
        return self.state.get('asr', None) if self.state else None

    @property
    def conversation_id(self) -> str:
        return self.state.get('conversation_id', None) if self.state else None

    @property
    def coref(self) -> str:
        return self.state.get('coref', None) if self.state else None

    @property
    def intentdetection(self) -> str:
        return self.state.get('intentdetection', None) if self.state else None

    @property
    def next_responder(self) -> str:
        return self.state.get('next_responder', None) if self.state else None

    @property
    def nounphrases(self) -> str:
        return self.state.get('nounphrases', None) if self.state else None

    @property
    def phonemes(self) -> str:
        return self.state.get('phonemes', None) if self.state else None

    @property
    def punctuation(self) -> str:
        return self.state.get('punctuation', None) if self.state else None

    @property
    def request_type(self) -> str:
        return self.state.get('request_type', None) if self.state else None

    @property
    def resume_responder(self) -> str:
        return self.state.get('resume_responder', None) if self.state else None

    @property
    def resume_task(self) -> str:
        return self.state.get('resume_task', None) if self.state else None

    @property
    def sentiment_score(self) -> str:
        return self.state.get('sentiment_score', None) if self.state else None

    @property
    def slots(self) -> str:
        return self.state.get('slots', None) if self.state else None

    @property
    def user_id(self) -> str:
        return self.state.get('user_id', None) if self.state else None
